/**
 * FLL - Level 3
 *
 * Project: IKI Write
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Provides the common type structures.
 *
 * This is auto-included and should not need to be explicitly included.
 */
#ifndef _iki_write_common_type_h
#define _iki_write_common_type_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * The IKI write main program cache.
 *
 * Properties:
 *   - buffer:  A buffer used during processing the file.
 *   - content: A buffer used to hold a Content during processing.
 *   - iki:     A buffer used for writing the IKI during processing.
 *   - objects: A buffer used to hold Objects during processing.
 */
#ifndef _di_iki_write_setting_t_
  typedef struct {
    f_string_dynamic_t buffer;
    f_string_dynamic_t content;
    f_string_dynamic_t iki;
    f_string_dynamics_t objects;
  } iki_write_cache_t;

  #define iki_write_cache_t_initialize \
    { \
      f_string_dynamic_t_initialize, \
      f_string_dynamic_t_initialize, \
      f_string_dynamic_t_initialize, \
      f_string_dynamics_t_initialize, \
    }
#endif // _di_iki_write_cache_t_

/**
 * The IKI write callbacks.
 *
 * Properties:
 *   - load_objects_content_pipe: Load the Objects and Content from the pipe.
 *
 *   - print_help: Print the main help message.
 *
 *   - process_objects_content: Process an Objects and Content set.
 */
#ifndef _di_iki_write_callback_t_
  typedef struct {
    void (*load_objects_content_pipe)(void * const main, const f_range_t range, uint8_t * const pipe_state);

    void (*print_help)(fl_print_t * const print);

    void (*process_objects_content)(void * const main, const f_string_statics_t objects, const f_string_static_t content);
  } iki_write_callback_t;

  #define iki_write_callback_t_initialize \
    { \
      0, \
      0, \
      0, \
    }
#endif // _di_iki_write_callback_t_

/**
 * The IKI write main program settings.
 *
 * This is passed to the program-specific main entry point to designate program settings.
 * These program settings are often processed from the program arguments (often called the command line arguments).
 *
 * Properties:
 *   - flag: Flags passed to the main function.
 *
 *   - status_signal: A status used eclusively by the threaded signal handler.
 *   - state:         The state data used when processing the FSS data.
 *
 *   - quote: This holds the quote used during processing.
 *
 *   - contents: An array of content passed as values to the "--content" parameter and must match the length of objects.
 *   - objectss: An array of an array of objects passed as values to the "--object" parameter.
 */
#ifndef _di_iki_write_setting_t_
  typedef struct {
    uint16_t flag;

    f_status_t status_signal;
    f_state_t state;

    f_string_static_t quote;

    f_string_dynamics_t contents;
    f_string_dynamicss_t objectss;
  } iki_write_setting_t;

  #define iki_write_setting_t_initialize \
    { \
      iki_write_main_flag_none_d, \
      F_okay, \
      f_state_t_initialize, \
      f_string_static_t_initialize, \
      f_string_dynamics_t_initialize, \
      f_string_dynamicss_t_initialize, \
    }
#endif // _di_iki_write_setting_t_

/**
 * The main program data as a single structure.
 *
 * Properties:
 *   - cache:    The cache data.
 *   - callback: The callbacks.
 *   - program:  The main program data.
 *   - setting:  The settings data.
 */
#ifndef _di_iki_write_main_t_
  typedef struct {
    iki_write_cache_t cache;
    iki_write_callback_t callback;
    fll_program_data_t program;
    iki_write_setting_t setting;
  } iki_write_main_t;

  #define iki_write_main_t_initialize \
    { \
      iki_write_cache_t_initialize, \
      iki_write_callback_t_initialize, \
      fll_program_data_t_initialize, \
      iki_write_setting_t_initialize, \
    }
#endif // _di_iki_write_main_t_

/**
 * Delete the program main cache data.
 *
 * @param cache
 *   The program main cache data.
 *
 *   Must not be NULL.
 *
 * @see f_memory_array_resize()
 */
#ifndef _di_iki_write_cache_delete_
  extern void iki_write_cache_delete(iki_write_cache_t * const cache);
#endif // _di_iki_write_cache_delete_

/**
 * De-allocate main program data.
 *
 * @param main
 *   The main program data.
 *
 *   Must not be NULL.
 *
 *   This does not alter main.setting.state.status.
 *
 * @see fll_program_data_delete()
 * @see iki_write_cache_delete()
 * @see iki_write_setting_delete()
 */
#ifndef _di_iki_write_main_delete_
  extern void iki_write_main_delete(iki_write_main_t * const main);
#endif // _di_iki_write_main_delete_

/**
 * Delete the program main setting data.
 *
 * @param setting
 *   The program main setting data.
 *
 *   Must not be NULL.
 *
 *   This does not alter setting.state.status.
 *
 * @see f_memory_arrays_resize()
 */
#ifndef _di_iki_write_setting_delete_
  extern void iki_write_setting_delete(iki_write_setting_t * const setting);
#endif // _di_iki_write_setting_delete_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _iki_write_common_type_h
